@@ -3,9 +3,5 @@ use strict;
 use warnings;
 use Log::Handler;
 
-my $log = Log::Handler->new(
-   filename => \*STDOUT,
-   newline  => 1
-);
-
+my $log = Log::Handler->new();
 $log->trace("CALLER INFORMATIONS:");
